@@ -72,7 +72,11 @@ class MockServerWrapper(Thread):
     
     def require_request(self):
         return self.requests.get(block=False)
-    
+
+    def wait_until_request_received(self):
+        req = self.requests.get()
+        self.requests.put(req)
+
     def should_have_requests(self, count):
         if self.requests.qsize() != count:
             rs = []
