@@ -9,6 +9,7 @@ from ldclient.diagnostics import create_diagnostic_id, _DiagnosticAccumulator
 from ldclient.event_processor import DefaultEventProcessor
 from ldclient.util import log
 from testing.http_util import start_server, BasicResponse
+from testing.proxy_test_util import do_proxy_tests
 from testing.stub_util import MockResponse, MockHttp
 
 
@@ -277,6 +278,33 @@ def test_event_can_be_both_tracked_and_debugged():
         check_feature_event(output[2], e, True, user)
         check_summary_event(output[3])
 
+def test_debug_mode_does_not_expire_if_both_client_time_and_server_time_are_before_expiration_time():
+    with DefaultTestProcessor() as ep:
+        # Pick a server time that slightly different from client time
+        server_time = now() + 1000
+
+        # Send and flush an event we don't care about, just to set the last server time
+        mock_http.set_server_time(server_time)
+        ep.send_event({ 'kind': 'identify', 'user': { 'key': 'otherUser' }})
+        flush_and_get_events(ep)
+
+        # Now send an event with debug mode on, with a "debug until" time that is further in
+        # the future than both the client time and the server time
+        debug_until = server_time + 10000
+        e = {
+            'kind': 'feature', 'key': 'flagkey', 'version': 11, 'user': user,
+            'variation': 1, 'value': 'value', 'default': 'default',
+            'trackEvents': False, 'debugEventsUntilDate': debug_until
+        }
+        ep.send_event(e)
+
+        # Should get a summary event only, not a full feature event
+        output = flush_and_get_events(ep)
+        assert len(output) == 3
+        check_index_event(output[0], e, user)
+        check_feature_event(output[1], e, True, user)  # debug event
+        check_summary_event(output[2])
+
 def test_debug_mode_expires_based_on_client_time_if_client_time_is_later_than_server_time():
     with DefaultTestProcessor() as ep:
         # Pick a server time that is somewhat behind the client time
@@ -558,52 +586,13 @@ def test_does_not_block_on_full_inbox():
         assert message1.param == event1
         assert had_no_more
 
-def test_can_use_http_proxy_via_environment_var(monkeypatch):
-    with start_server() as server:
-        monkeypatch.setenv('http_proxy', server.uri)
-        config = Config(sdk_key = 'sdk-key', events_uri = 'http://not-real', diagnostic_opt_out = True)
-        _verify_http_proxy_is_used(server, config)
-
-def test_can_use_https_proxy_via_environment_var(monkeypatch):
-    with start_server() as server:
-        monkeypatch.setenv('https_proxy', server.uri)
-        config = Config(sdk_key = 'sdk-key', events_uri = 'https://not-real', diagnostic_opt_out = True)
-        _verify_https_proxy_is_used(server, config)
-
-def test_can_use_http_proxy_via_config():
-    with start_server() as server:
-        config = Config(sdk_key = 'sdk-key', events_uri = 'http://not-real', http_proxy=server.uri, diagnostic_opt_out = True)
-        _verify_http_proxy_is_used(server, config)
-
-def test_can_use_https_proxy_via_config():
-    with start_server() as server:
-        config = Config(sdk_key = 'sdk-key', events_uri = 'https://not-real', http_proxy=server.uri, diagnostic_opt_out = True)
-        _verify_https_proxy_is_used(server, config)
-
-def _verify_http_proxy_is_used(server, config):
-    server.for_path(config.events_uri + '/bulk', BasicResponse(200))
-    with DefaultEventProcessor(config) as ep:
-        ep.send_event({ 'kind': 'identify', 'user': user })
-        ep.flush()
-        ep._wait_until_inactive()
-
-        # For an insecure proxy request, our stub server behaves enough like the real thing to satisfy the
-        # HTTP client, so we should be able to see the request go through. Note that the URI path will
-        # actually be an absolute URI for a proxy request.
-        req = server.require_request()
-        assert req.method == 'POST'
-
-def _verify_https_proxy_is_used(server, config):
-    server.for_path(config.events_uri + '/bulk', BasicResponse(200))
-    with DefaultEventProcessor(config) as ep:
-        ep.send_event({ 'kind': 'identify', 'user': user })
-        ep.flush()
-        ep._wait_until_inactive()
-
-        # Our simple stub server implementation can't really do HTTPS proxying, so the request will fail, but
-        # it can still record that it *got* the request, which proves that the request went to the proxy.
-        req = server.require_request()
-        assert req.method == 'CONNECT'
+def test_http_proxy(monkeypatch):
+    def _event_processor_proxy_test(server, config, secure):
+        with DefaultEventProcessor(config) as ep:
+            ep.send_event({ 'kind': 'identify', 'user': user })
+            ep.flush()
+            ep._wait_until_inactive()
+    do_proxy_tests(_event_processor_proxy_test, 'POST', monkeypatch)
 
 def verify_unrecoverable_http_error(status):
     with DefaultTestProcessor(sdk_key = 'SDK_KEY') as ep:
